{% load i18n %}
{% for entry in answer_entries %}
  <div class="quiz-answer{% if entry.id and not entry.selected %} quiz-answer--not-selected{% endif %}">
    <input
      type="checkbox"
      id="wa_sum_{{ fieldset_index }}_{{ forloop.counter }}"
      {% if not entry.id or entry.selected %}checked{% endif %}
      disabled
    />
    <label for="wa_sum_{{ fieldset_index }}_{{ forloop.counter }}">
      <div class="answer">{% if entry.id %}{{ entry.label|safe }}{% else %}{{ entry|safe }}{% endif %}</div>
    </label>
  </div>
{% empty %}
  <p>
    <i>{% translate "You haven't selected any answer." %}</i>
  </p>
{% endfor %}
