# Generated by Django 3.2.24 on 2026-03-31 12:00

# Django
from django.db import migrations, models

# Third party
import ckeditor_uploader.fields


class Migration(migrations.Migration):

    dependencies = [
        ("django_easy_quiz", "0009_moreinfoquestion_is_required"),
    ]

    operations = [
        migrations.AddField(
            model_name="weightedanswersquiz",
            name="show_answers_corrections_text",
            field=models.BooleanField(
                default=False, verbose_name="Show answers explanatory text in summary"
            ),
        ),
        migrations.AddField(
            model_name="weightedanswersquizquestion",
            name="question_explanation_text",
            field=ckeditor_uploader.fields.RichTextUploadingField(
                blank=True, null=True, verbose_name="Explanation text for question"
            ),
        ),
    ]
