# Generated by Django 3.2.22 on 2026-03-31 14:19

# Django
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("django_easy_quiz", "0011_auto_20260331_1156"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="interpretationquiz", name="conclusion_max_1"
        ),
        migrations.RemoveField(
            model_name="interpretationquiz", name="conclusion_max_2"
        ),
        migrations.RemoveField(
            model_name="interpretationquiz", name="conclusion_max_3"
        ),
        migrations.RemoveField(
            model_name="interpretationquiz", name="conclusion_max_4"
        ),
        migrations.RemoveField(
            model_name="interpretationquizquestion", name="answer_1"
        ),
        migrations.RemoveField(
            model_name="interpretationquizquestion", name="answer_2"
        ),
        migrations.RemoveField(
            model_name="interpretationquizquestion", name="answer_3"
        ),
        migrations.RemoveField(
            model_name="interpretationquizquestion", name="answer_4"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_1"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_10"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_2"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_3"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_4"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_5"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_6"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_7"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_8"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="answer_9"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_1"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_10"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_2"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_3"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_4"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_5"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_6"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_7"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_8"
        ),
        migrations.RemoveField(
            model_name="weightedanswersquizquestion", name="points_answer_9"
        ),
    ]
